#!/usr/bin/env python

import sys

from django.conf import settings
from django.core.management import execute_from_command_line


if not settings.configured:
    params = dict(
        LOGGING = {
            'version': 1,
            'disable_existing_loggers': False,
            'handlers': {
                'console': {
                    'class': 'logging.StreamHandler',
                },
            },
            'loggers': {
                'wagtailsystemtext': {
                    'handlers': ['console'],
                    'level': 'ERROR',
                    'propagate': True,
                },
            },
        },
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
            }
        },
        INSTALLED_APPS=[
            'django.contrib.contenttypes',
            'django.contrib.auth',
            'django.contrib.sites',
            'wagtail.wagtailcore',
            'wagtail.wagtailsites',
            'wagtail.wagtailusers',
            'wagtail.wagtailimages',
            'taggit',
            'wagtailsystemtext',
            "tests",
        ],
        SYSTEMTEXT_CACHE_PREFIX='wagtailsystemtext_test',
        MIDDLEWARE_CLASSES=[],
        ROOT_URLCONF='tests.urls',
    )

    settings.configure(**params)


def runtests():
    argv = sys.argv[:1] + ["test"] + sys.argv[1:]
    execute_from_command_line(argv)


if __name__ == "__main__":
    runtests()



from wagtail.contrib.modeladmin.options import (
    ModelAdmin,
    modeladmin_register,
)
from wagtail.contrib.modeladmin.views import EditView, CreateView

from wagtailsystemtext import app_settings
from wagtailsystemtext.utils import get_admin_site, fill_cache
from wagtailsystemtext.models import SystemString


class SystemStringEditView(EditView):
    def form_valid(self, form):
        redirect = super(SystemStringEditView, self).form_valid(form)
        self.instance.modified = True
        self.instance.save()

        if app_settings.SYSTEMTEXT_REBUILD_ON_SAVE:
            site = form.cleaned_data['site']
            fill_cache(site)

        return redirect


class SystemStringCreateView(CreateView):
    def form_valid(self, form):
        redirect = super(SystemStringCreateView, self).form_valid(form)

        if app_settings.SYSTEMTEXT_REBUILD_ON_SAVE:
            site = form.cleaned_data['site']
            fill_cache(site)

        return redirect


class SystemStringAdmin(ModelAdmin):
    model = SystemString
    menu_label = 'System Text'
    menu_order = 200
    list_display = ('modified', 'identifier', 'string', 'group', 'site')
    search_fields = ('identifier', 'string')
    add_to_settings_menu = True
    create_view_class = SystemStringCreateView
    edit_view_class = SystemStringEditView

    def get_queryset(self, request):
        qs = super(SystemStringAdmin, self).get_queryset(request)

        site = get_admin_site()
        if site:
            qs = qs.filter(site=site)

        return qs


modeladmin_register(SystemStringAdmin)



import six

try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

from django.core.cache import cache
from django.utils.functional import lazy

from wagtailsystemtext.models import SystemString
from wagtailsystemtext import app_settings


_thread_locals = local()


def _cleanup():
    if hasattr(_thread_locals, 'site'):
        del _thread_locals.site

    if hasattr(_thread_locals, 'index'):
        del _thread_locals.index


def set_site(site):
    _thread_locals.site = site


def get_site():
    return getattr(_thread_locals, 'site', None)


def set_admin_site(site):
    _thread_locals.admin_site = site


def get_admin_site():
    return getattr(_thread_locals, 'admin_site', None)


def in_cache(site):
    return cache_key(site) in cache


def cache_key(site):
    return '{}:{}'.format(app_settings.SYSTEMTEXT_CACHE_PREFIX, site.pk)


def get_from_cache(site):
    return cache.get(cache_key(site))


def fill_cache(site):
    string_qs = SystemString.objects.filter(site=site)
    strings = {u'{}:{}'.format(x.group, x.identifier): (x.string, x.modified)
               for x in string_qs}

    cache.set(cache_key(site), strings, app_settings.SYSTEMTEXT_CACHE_EXPIRY)


def preload(site):
    index = get_index()
    strings = get_from_cache(site)
    index['site_{}'.format(site.pk)] = strings

    _thread_locals.index = index


def get_index():
    if not getattr(_thread_locals, 'index', None):
        return {}

    return _thread_locals.index


def current_strings():
    values = get_index()
    site = get_site()
    key = 'site_{}'.format(site.pk)

    if key not in values:
        return {}

    return values[key]


def systemtext(identifier, group=SystemString.DEFAULT_GROUP, default=None):
    site = get_site()

    if not site:
        return default if default else ''

    strings = current_strings()
    value = identifier
    key = u'{}:{}'.format(group, identifier)

    if key not in strings:
        instance = SystemString.objects.get_or_create(
            identifier=identifier,
            group=group,
            site=site,
            modified=False,
        )

        return default if default else value

    string, modified = strings[key]

    if not modified:
        return default if default else string

    if not string and default and app_settings.SYSTEMTEXT_USE_DEFAULT_ON_EMPTY:
        return default

    return string


systemtext_lazy = lazy(systemtext, six.text_type)



from django.core.management.base import BaseCommand
from wagtail.wagtailcore.models import Site

from wagtailsystemtext.models import SystemString


class Command(BaseCommand):
    """
    Add system text to Wagtail websites
    """
    def add_arguments(self, parser):
        parser.add_argument('--identifier', required=True)
        parser.add_argument('--group', default=SystemString.DEFAULT_GROUP)
        parser.add_argument('--site_id', type=int, default=-1)

    def handle(self, *args, **options):
        identifier = options['identifier']
        group = options['group']
        site_id = options['site_id']

        sites = Site.objects.all()
        if site_id != -1:
            sites = sites.filter(pk=site_id)

        for site in sites:
            if SystemString.objects.filter(identifier=identifier,
                                           group=group,
                                           site=site).exists():

                self.stdout.write(u'{}:{} on {} already exist'.format(
                    identifier, group,
                    site))
                continue

            self.stdout.write(u'Adding: {}:{} to {}'.format(identifier, group,
                                                            site))
            SystemString.objects.create(
                identifier=identifier,
                group=group,
                site=site,
            )

        self.stdout.write('Done!')



from django.core.management.base import BaseCommand
from wagtail.wagtailcore.models import Site

from wagtailsystemtext.models import SystemString


class Command(BaseCommand):
    """
    List system text from Wagtail websites
    """
    def add_arguments(self, parser):
        parser.add_argument('--sort_field', default='identifier')
        parser.add_argument('--site_id', type=int, default=-1)

    def handle(self, *args, **options):
        sort_field = options['sort_field']
        site_id = options['site_id']

        sites = Site.objects.all()
        if site_id != -1:
            sites = sites.filter(pk=site_id)

        strings = SystemString.objects.filter(site__in=sites)\
            .order_by(sort_field)

        for string in strings:
            self.stdout.write(u'{}:{} on {}'.format(string.identifier,
                                                    string.group,
                                                    string.site))



import logging

from django.utils.deprecation import MiddlewareMixin

from wagtailsystemtext.utils import (
    in_cache,
    fill_cache,
    set_site,
    preload,
)


logger = logging.getLogger(__name__)


class SiteSystemTextMiddleware(MiddlewareMixin):
    def process_request(self, request):
        if not hasattr(request, 'site'):
            logger.error('Request is missing site instance, have you added '
                         'wagtail.wagtailcore.middleware.SiteMiddleware '
                         'to your middleware classes?')

        site = request.site

        if not site:
            logger.error('Site is None on middleware request')
            return

        set_site(request.site)

        if not in_cache(site):
            fill_cache(site)

        preload(site)



import re
import os
import logging

from django.core.management.base import BaseCommand
from wagtail.wagtailcore.models import Site

from wagtailsystemtext.models import SystemString


logger = logging.getLogger(__name__)


API_PTRNS = (
    re.compile("_st\((.*?)\)"),
    re.compile("systemtext\((.*?)\)"),
)

TMPL_PTRNS = re.compile("{%.systemtext(.*)%}")


def get_files(path):
    matched_files = []

    for root, dirs, files in os.walk(path):
        for file_ in files:
            if file_.endswith('.py'):
                matched_files.append(os.path.join(root, file_))
            if file_.endswith('.html'):
                matched_files.append(os.path.join(root, file_))

    return matched_files


def parse_api(result):
    if not result:
        return None

    value = result.group(1)
    value = value.replace("'", "")
    value = value.replace("\"", "")
    value = value.split(",")
    value = [x.strip() for x in value]
    value = value[:2]

    if len(value) == 1:
        value.append(SystemString.DEFAULT_GROUP)

    identifier = value[0]
    group = value[1]

    if not re.search('^\w*$', identifier) or not \
            re.search('^\w*$', group):
        return None

    return (identifier, group)


def parse_tmpl(result):
    if not result:
        return None

    value = result.group(1)
    value = value.strip()
    value = value.split(" ")
    value = [x.replace("'", "") for x in value]
    value = [x.replace("\"", "") for x in value]
    value = [x.strip() for x in value]

    identifier = value[0]
    remaining = value[1:]

    group = SystemString.DEFAULT_GROUP

    while remaining:
        option = remaining.pop(0)
        if option == 'group':
            group = remaining.pop(0)

    if not re.search('^\w*$', identifier) or not \
            re.search('^\w*$', group):
        return None

    return (identifier, group)


def find_strings(path):
    strings = []

    for file_ in get_files(path):
        content = open(file_).read()
        strings += [parse_tmpl(x) for x in re.finditer(TMPL_PTRNS, content)]

        for ptn in API_PTRNS:
            strings += [parse_api(x) for x in re.finditer(ptn, content)]

    strings = filter(None, strings)
    return strings


class Command(BaseCommand):
    """
    Find and add system text to the various websites
    """
    def add_arguments(self, parser):
        parser.add_argument('--path', required=False)
        parser.add_argument('--dryrun', required=False)
        parser.add_argument('--limit', type=int, default=-1)
        parser.add_argument('--force', type=bool, default=False)

    def handle(self, *args, **options):
        path = options['path']
        dryrun = options['dryrun']
        limit = options['limit']
        force = options['force']

        if not path:
            path = os.getcwd()

        self.stdout.write('Using path "{}"'.format(path))

        strings = find_strings(path)
        sites = Site.objects.all()

        if limit != -1:
            strings = strings[:limit]

        self.stdout.write('Found {} strings'.format(len(strings)))

        if not force:
            proceed = raw_input('Proceed adding strings? (y/n): ')
            if proceed != 'y':
                return

        for string in strings:
            identifier, group = string
            self.stdout.write(u'Adding: {}:{}'.format(identifier, group))

            if dryrun:
                continue

            for site in sites:
                SystemString.objects.get_or_create(
                    identifier=identifier,
                    group=group,
                    site=site,
                )

        self.stdout.write('Done!')



from django.core.management.base import BaseCommand
from wagtail.wagtailcore.models import Site

from wagtailsystemtext.models import SystemString


class Command(BaseCommand):
    """
    Remove system text from all Wagtail websites
    """
    def add_arguments(self, parser):
        parser.add_argument('--identifier', required=True)
        parser.add_argument('--group', default=SystemString.DEFAULT_GROUP)
        parser.add_argument('--site_id', type=int, default=-1)

    def handle(self, *args, **options):
        identifier = options['identifier']
        group = options['group']
        site_id = options['site_id']

        sites = Site.objects.all()
        if site_id != -1:
            sites = sites.filter(pk=site_id)

        for site in sites:
            if not SystemString.objects.filter(identifier=identifier,
                                               group=group,
                                               site=site).exists():

                self.stdout.write(u'{}:{} on {} already removed'.format(
                    identifier, group,
                    site))

                continue

            self.stdout.write(u'Removing: {}:{} from {}'.format(identifier,
                                                                group,
                                                                site))

            SystemString.objects.filter(identifier=identifier, group=group,
                                        site=site).delete()

        self.stdout.write('Done!')



from django.core.management.base import BaseCommand
from wagtail.wagtailcore.models import Site

from wagtailsystemtext.models import SystemString


class Command(BaseCommand):
    """
    Sync system text to make sure all Wagtail sites contains the same text
    """
    def handle(self, *args, **options):
        strings = SystemString.objects.values_list('identifier', 'group')\
            .distinct()

        sites = Site.objects.all()
        for string in strings:
            identifier, group = string
            for site in sites:
                if SystemString.objects.filter(identifier=identifier,
                                               group=group,
                                               site=site).exists():
                    continue

                self.stdout.write(u'Syncing: {}:{} to {}'.format(identifier,
                                                                 group,
                                                                 site))
                SystemString.objects.create(
                    identifier=identifier,
                    group=group,
                    site=site,
                )

        self.stdout.write('Done!')
